
--[[

Hardon Collider Example for Gideros
by Cliff Earl
Antix Software
March 2016

Example project that  loads a tiled  map, renders it, and  creates collision 
shapes from one of its layers and adds  those to a Hardon Collider world. It 
also updates the Collision world which detects collisions and performs basic 
collision resolution  by pushing  the draggable  ball shape out of  any wall 
shape that it collides with. 

Hardon Collider is a Love2d library that detects collisions between circles, 
polygons (convex and concave), and points. You just  add collision shapes to 
the collider, update it every frame, and  then respond to  collisions via  a 
callback fundtion. The version of Hardon Collider provided with this example 
is a version that has  been modified to work with Gideros. You can read more 
about Hardon Collider at https://vrld.github.io/HardonCollider/ 

Besides  the Hardon Collider library  (and its many associated  lua files) 1 
class is provided, Level...

Level
A Sprite extension that can load tiled maps. It creates collision shapes for 
objects in objectlayers of the type "solid". These collision shapes are then 
added to the Hardon Collioder  world. It can also creates  a graphical shape 
(Gideros Shape) for each collision shape it creates. See  Level.lua for more 
information.

--]]

--require("mobdebug").start() -- ONLY REQUIRED FOR ZEROBRANE

require("json")

local currentTimer = 0

local COLLIDER = HC.new(100) -- CREATE NEW COLLISION WORLD

local Level = Level.new({collider = COLLIDER, createShapes = true}) -- LOAD A MAP
Level:loadMap("level0000")
Level:toFront()

local x, y = Level:getPlayerStart() -- CREATE A BALL OBJECT
local ball = Bitmap.new(Texture.new("images/ball.png", true))
ball:setAnchorPoint(0.5, 0.5)
ball.shape = COLLIDER:addCircle(x, y, ball:getWidth() * 0.5)
ball.position = ({x = 0, y = 0})
ball:setPosition(x, y)
stage:addChild(ball)

local arrow1 = Bitmap.new(Texture.new("images/arrow.png", true)) -- SOME RANDOM ARROW
arrow1:setAnchorPoint(0.5, 0.5)
arrow1:setRotation(0)
arrow1:setPosition(x, y)
arrow1:setVisible(false)
stage:addChild(arrow1)

local DEBUG = TextField.new(nil,"") -- JUST TO SOW A FEW VARS
DEBUG:setPosition(0, 64)
stage:addChild(DEBUG)

local function onCollision(dt, shape_a, shape_b, mtv_x, mtv_y) -- CALLED WHEN COLLISION DETECTED
    local floor, radToDeg, atan2 = math.floor, 180/ math.pi, math.atan2
    
    local ball_shape, tileshape -- sort out which one our ball shape is
    if shape_a == ball.shape and shape_b.type == "tile" then
      ball_shape = shape_a
    elseif shape_b == ball.shape and shape_a.type == "tile" then
      ball_shape = shape_b
    else
      return -- NONE OF THE TWO SHAPES IS A TILE :(
    end
    
    local angle = ( ( atan2(mtv_y, mtv_x) * radToDeg ) -90) % 360 -- ANGLE OF COLLISION
  
    ball_shape:move(mtv_x, 0) -- CORRECT COLLISION SHAPE X, Y
    ball_shape:move(0, mtv_y)
    
    local fx, fy = ball_shape:center() -- CORRECT BALL X, Y
    ball.position.x = fx
    ball.position.y = fy
    ball:setPosition(fx, fy)
    ball:setColorTransform(1, 0, 0, 1)
    
    arrow1:setVisible(true)
    arrow1:setPosition(fx, fy)
    arrow1:setRotation(angle)

    DEBUG:setText("  angle="..floor(angle)..", mtv_x="..floor(mtv_x)..", mtv_y="..floor(mtv_y))
end

local function screenTouched(e) -- TELEPORT THE BALL TO WHERE THE SCREEN IS TOUCHED
  local x, y = e.touch.x, e.touch.y
  ball.shape:moveTo(x, y)
  ball:setPosition(x, y)
  ball:setColorTransform(1, 1, 1, 1)

  arrow1:setVisible(false)
  DEBUG:setText("")
end

local function onEnterFrame(event) -- MAIN GAME LOOP
	local timer = os.timer()
	local dt = timer - currentTimer
  
  COLLIDER:update(dt) -- UPDATE COLLISION WORLD
end

COLLIDER:setCallbacks(onCollision) --SET COLLISION WORLD CALLBACK

stage:addEventListener(Event.ENTER_FRAME, onEnterFrame)
stage:addEventListener(Event.TOUCHES_BEGIN, screenTouched)
stage:addEventListener(Event.TOUCHES_MOVE, screenTouched)
